using System;

using Lazy;

namespace Lazy.Example.EconomicAnalysis
{
	/// <summary>
	/// Descripcin breve de Class1.
	/// </summary>

	[Lazy]
	public class Demand {

		// Known market
		private double price;
		private long   quantity;
		// Demand elasticity (deltaQ/deltaP)
		private double elasticity;
		// Linear demand
		private double maxPrice;
		private long   maxMarket;
		private double slope;		// -maxPrice/maxMarket



		// Properties

		public double MaximumPrice {
			get { return maxPrice; }
			set { 
				maxPrice = value;
				if (maxMarket>0)
					slope = -maxPrice/maxMarket; 
			}
		}

		public long PotentialMarketSize {
			get { return maxMarket; }
			set { 
				maxMarket = value;
				if ((maxMarket>0)&&(maxPrice>0)) 
					slope = -maxPrice/maxMarket; 
			}
		}

		public double FixedElasticity {
			set {
				elasticity = value;
				if (elasticity<0)
					elasticity = -elasticity;
			}
		}


		public void KnownMarket (double price, long quantity) {
			this.price = price;
			this.quantity = quantity;		
		}

		// Price

		public virtual double Price (long quantity) {
			return 0;
		}

		[AlternativeImplementation("Price")]
		protected double PriceFromKnownMarket (long quantity) {
			return price;
		}

		[AlternativeImplementation("Price")]
		protected double PriceFromLinearDemand (long quantity) {

			return price = maxPrice + quantity*slope;
		}

		[AlternativeImplementation("Price")]
		protected double PriceFromElasticity (long quantity) {
			double deltaQ = (quantity-this.quantity)/(double)(this.quantity);

			return price*(1-deltaQ/elasticity);
		}

		// Demanded quantity

		public virtual long Quantity (double price) {		
			return 0;
		}

		[AlternativeImplementation("Quantity")]
		protected long QuantityFromKnownMarket (double price) {

			if (price<=this.price) {
				return this.quantity;
			} else {
				return 0;
			}
		}

		[AlternativeImplementation("Quantity")]
		protected long QuantityFromLinearDemand (double price) {

			if (price<maxPrice) {
				// return (long) (price-maxPrice)/slope;
				return (long) (maxMarket*(1-price/maxPrice));
			} else 
				return 0;
		}

		[AlternativeImplementation("Quantity")]
		protected long QuantityFromElasticity (double price) {
			
			double deltaP = (price-this.price)/(this.price);
			long   demand = (long) (quantity*(1-deltaP*elasticity));

			if (demand<0)
				demand = 0;

			return demand;
		}

		// Elasticity

		public double Elasticity (long quantity) {
			return elasticity;
		}

		[AlternativeImplementation("Elasticity")]
		protected double ElasticityFromLinearDemand (long quantity) {
			double deltaQ = (1.1*quantity)/quantity;
			double deltaP = PriceFromLinearDemand((long)(1.1*quantity))/PriceFromLinearDemand(quantity);

			return deltaQ/deltaP;
		}
	}
}
