using System;

using Lazy;


namespace Lazy.Example
{
	/// <summary>
	/// Descripcin breve de Class1.
	/// </summary>
	
	[Lazy]
	public class Triangle {
		// Instance variables

		public double sideA;
		public double sideB;
		public double sideC;

		public double heightA;
		public double heightB;
		public double heightC;

		private double angleA;
		private double angleB;
		private double angleC;

		public double area;

		// Angles

		public double AngleA {
			get { return angleA; }
			set {
				angleA = value; 
				if (angleB>0) {
					angleC = Math.PI-angleA-angleB;
				} else if (angleC>0) {
					angleB = Math.PI-angleA-angleC;
				}
			}
		}

		public double AngleB {
			get { return angleB; }
			set {
				angleB = value; 
				if (angleA>0) {
					angleC = Math.PI-angleB-angleA;
				} else if (angleC>0) {
					angleA = Math.PI-angleB-angleC;
				}
			}
		}

		public double AngleC {
			get { return angleC; }
			set {
				angleC = value; 
				if (angleB>0) {
					angleA = Math.PI-angleC-angleB;
				} else if (angleA>0) {
					angleB = Math.PI-angleC-angleA;
				}
			}
		}

		
		// 11 ways to compute the area of a triangle

		// Given area

		public virtual double Area () {
			return area;
		}

		// 1 side + height
		
		[AlternativeImplementation("Area")]
		public double AreaFromHeightA () {
			return sideA*heightA/2;
		}

		[AlternativeImplementation("Area")]
		public double AreaFromHeightB () {
			return sideB*heightB/2;
		}

		[AlternativeImplementation("Area")]
		public double AreaFromHeightC () {
			return sideC*heightC/2;
		}

		// 1 side + 2 angles (3rd is implicit)

		[AlternativeImplementation("Area")]
		public double AreaFromSideA () {

			double h = sideA*Math.Sin(angleB)*Math.Sin(angleC)/Math.Sin(angleA);

			return sideA*h/2;
		}

		[AlternativeImplementation("Area")]
		public double AreaFromSideB () {

			double h = sideB*Math.Sin(angleA)*Math.Sin(angleC)/Math.Sin(angleB);

			return sideB*h/2;
		}

		[AlternativeImplementation("Area")]
		public double AreaFromSideC () {

			double h = sideC*Math.Sin(angleA)*Math.Sin(angleB)/Math.Sin(angleC);

			return sideC*h/2;
		}

		// 2 sides + 1 angle

		[AlternativeImplementation("Area")]
		public double AreaFrom2SidesA () {

			return sideB*sideC*Math.Sin(angleA)/2;
		}

		[AlternativeImplementation("Area")]
		public double AreaFrom2SidesB () {

			return sideA*sideC*Math.Sin(angleB)/2;
		}

		[AlternativeImplementation("Area")]
		public double AreaFrom2SidesC () {

			return sideA*sideB*Math.Sin(angleC)/2;
		}

		// 3 sides

		[AlternativeImplementation("Area")]
		public double AreaFrom3Sides () {

			double T = (sideA+sideB+sideC)/2;

			return Math.Sqrt(T*(T-sideA)*(T-sideB)*(T-sideC));
		}
			




	}
}
